#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include <libpq-fe.h>

void
exit_nicely(PGconn *conn)
{
	PQfinish(conn);
	exit(1);
}

int
main(void)
{
	const char		fname[] = "image.jpg";
	FILE			*fp;
	PGconn			*conn;
	PGresult		*res;
	struct	stat	 st;
	char			*buf;
	const int		 l = 1;
	const char		*param_vals[l];
	int				 param_lens[l];
	int				 param_fmts[l];

	conn = PQconnectdb("dbname=test");
	if (PQstatus(conn) != CONNECTION_OK)
	{
		fprintf(stderr, "Error: PQconnectdb: %s", PQerrorMessage(conn));
		exit_nicely(conn);
	}

	if(stat(fname, &st) < 0)
	{
		perror("stat");
		exit_nicely(conn);
	}
	
	buf = malloc(st.st_size);
	if (!buf)
	{
		perror("malloc");
		exit_nicely(conn);
	}

	fp = fopen("image.jpg", "r");
	if (!fp)
	{
		free(buf);
		perror("fopen");
		exit_nicely(conn);
	}
	fread(buf, sizeof(char), st.st_size, fp);
	fclose(fp);

	param_vals[0] = buf;
	param_lens[0] = st.st_size;
	param_fmts[0] = 1;
	res = PQexecParams(conn, "INSERT INTO file_arch (data) VALUES ($1)",
					   l, NULL, param_vals, (const int *) param_lens,
					   (const int *) param_fmts, 1);
	if (PQresultStatus(res) != PGRES_COMMAND_OK)
	{
		fprintf(stderr, "INSERT failed: %s", PQerrorMessage(conn));
		PQclear(res);
		free(buf);
		exit_nicely(conn);
	}
	
	PQclear(res);
	free(buf);

	return 0;
}
